/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WLINE2_H
#define WLINE2_H

#include "tbox.h"
#include "wLazy.h"
#include "wElement.h"
#include "wCacheLine2.h"

namespace tbox
{

/**
 * @brief Linear line finite element
 * @authors Romain Boman, Adrien Crovato
 */
class TBOX_API Line2 : public Element
{
#ifndef SWIG
private:
    inline static CacheLine2 &getCache(int n);

protected:
    virtual void buildSurfaceJ() override;
    virtual void buildGradientSurfaceJ() override;
    virtual void computeV() override;
    virtual void buildGradientV(int dim) override;
    virtual void computeNormal() override;
    virtual void computeGradientNormal() override;
    virtual std::vector<Eigen::Vector3d> computeTangents() const override;
    virtual std::vector<Eigen::Vector3d> buildTangents(size_t k) const override;
#endif

public:
    Line2(int n, Tag *_ptag, Tag *_etag, std::vector<Tag *> &_parts, std::vector<Node *> &nods);
    virtual ~Line2() {}
    virtual ElType type() const override { return ElType::LINE2; }

#ifndef SWIG
    virtual void initValues(bool isvol, int ordr = 2) override;
    virtual void initGradients() override;
    virtual void update() override;
    virtual Cache &getVCache() const override;
#endif
};

#include "wLine2.inl"

} // namespace tbox

#endif // WLINE2_H
