/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WMSHEXPORT_H
#define WMSHEXPORT_H

#include "tbox.h"
#include "wObject.h"
#include "wMshData.h"
#include <memory>

namespace tbox
{

/**
 * @brief Writer type
 * @authors Adrien Crovato
 */
enum class WrtType
{
    UNDEFINED = 0,
    GMSH = 1,
    VTK = 2
};

/**
 * @brief Base class to write mesh
 * @authors Adrien Crovato, Romain Boman
 */
class TBOX_API MshExport : public fwk::wSharedObject
{
public:
    std::shared_ptr<MshData> msh;
    bool binary;
    bool verb;

    MshExport(std::shared_ptr<MshData> _msh);
    virtual ~MshExport() {}
    virtual WrtType type() const { return WrtType::UNDEFINED; }

    virtual void save(std::string const &fname) const;
#ifndef SWIG
    virtual void save(std::string const &fname, Results const &r) const;
    virtual void saveList(std::string const &fname, Results const &r) const;
#endif
};

} // namespace tbox

#endif // WMSHEXPORT_H
