/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WNODE_H
#define WNODE_H

#include "tbox.h"
#include "wObject.h"
#include <vector>
#include <Eigen/Dense>

namespace tbox
{

/**
 * @brief Node data structure
 * @authors Romain Boman
 */
class TBOX_API Node : public fwk::wObject
{
public:
    int no;              ///< node number
    Eigen::Vector3d pos; ///< spatial position
    int row;             ///< row number in data vectors

    Node(int n, Eigen::Vector3d const &p, int r);
    Node();

#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif
};

} // namespace tbox

#endif // WNODE_H
